package TVProgram;

use IO::Socket;

BEGIN{
	$TVProgram::host='tv.rambler.ru';
	$TVProgram::dir='tvprogram'; #    "tvprogram"
	$TVProgram::chan='channels.txt';
	$TVProgram::time=time;
	use Time2Str;
	$TVProgram::timer=new Time2Str();
}

sub TVProgram::getDocument
{
	my$args={@_};
	my($host, $port, $page, $cset)=delete@$args{+qw'host port page cset'};
	$port||=80;
	$page||='/';
	$cset||='windows-1251';

	my $sock=new IO::Socket::INET(
		PeerAddr=>$host,
		PeerPort=>$port,
		Proto=>'tcp'
	);

	print $sock "GET $page HTTP/1.0\n",
					"Host: $host\n",
					"Accept-Charset: $cset\n\n";

	+$sock
}

sub TVProgram::getPage
{
	my$args={@_};
	my$time=delete$$args{time}||time;	# 
	my$chan=delete$$args{chan}||return; # 
	my$town=delete$$args{town}||94;		# . - - .
	my$uri=sprintf'/%d/index.html?channel_id=%d&d=%s',
					$town, $chan, $$TVProgram::timer{"_$time:Y-m-d"};

	my$param=delete$$args{param};
	$$param{host}||=delete$$args{host};
	$$param{page}=$uri;

	+&TVProgram::getDocument(%$param);
}

sub TVProgram::checkAnswer{
	my$sock=shift;
	return undef unless <$sock>=~/^HTTP\/(\d\.\d)\s+(\d{3})\s*(.*?)\s+$/o&&$2==200;
}

sub TVProgram::fstrgoto
{
	my$fh=shift;
	my$stop=shift||return;
	my@res;

	while(<$fh>)
	{
		chomp;
		last if &$stop($_);
		push@res,$_
	}

	+@res
}

sub TVProgram::fgoto
{
	my$fh=shift;
	my$stop=shift||return;

	while(<$fh>)
	{
		return 1 if &$stop($_);
	}

	+undef
}

sub TVProgram::skipHeaders{
	my$sock=shift;
	return undef unless &TVProgram::checkAnswer($sock);
	&TVProgram::fgoto($sock, sub{+$_[0]=~!/\S/o});
	+1
}

sub TVProgram::getHeaders{
	my$args={@_};
	my$sock=$$args{sock};
	my%header;

	return undef unless &TVProgram::checkAnswer($sock);

	while(<$sock>)
	{
		chomp;
		last unless /\S/o;
		next unless /^([\w\-]+)\:\s*(.*?)\s*$/o;
		$header{+$1}=$2;
	}

	+\%header
}

sub TVProgram::getItem
{
	my$page=shift;
	my$func=shift;
	my$cur;

	while(@$page)
	{
		return $cur if length($cur=&$func(shift@$page));
	}

	+undef
}

sub TVProgram::getOneProg
{
	my$page=shift;
	+{
		time=>&TVProgram::getItem($page, sub{+($_[0]=~/<font.+?>(.*?)<\/font/io)[0]}),
		title=>&TVProgram::getItem($page, sub{+($_[0]=~/<font.+?>(.*?)<\/font/io)[0]})
	}
}

sub TVProgram::getDirName{+sprintf$TVProgram::dir.'/%d',$_[0]}
sub TVProgram::getFileName{+&TVProgram::getDirName($_[0]).'/'.$_[1]}

sub TVProgram::load
{
	my$file=&TVProgram::getFileName(@_);
	my$program=[];
	my$fh;

	return undef unless -f$file;

	open$fh,'<'.$file;
		#flock $fh,4;
			while(<$fh>)
			{
				chomp;
				push@$program,+{
					time=>$1,
					title=>$2
				} if /^\s*(.+?)\s*\t\s*(.+)\s*/o
			}
		#flock $fh,8;
	close$fh;

	+$program
}

sub TVProgram::save
{
	my$chan=shift;
	my$date=shift;
	my$program=shift||return;
	my($fh, $cur);
	my$dir=sprintf"$TVProgram::dir/%d",$chan;
	mkdir $dir,0700 unless -d$dir;
	my$file=&TVProgram::getFileName($chan, $date);

	open$fh,'>'.$file;
		#flock $fh,2;
			foreach$cur(@$program)
			{
				$$cur{$_}=~s/[ \t\r\n]+/ /go foreach keys%$cur;
				print $fh $$cur{time},"\t",$$cur{title},"\n"
			}
		#flock $fh,8;
	close$fh;

	+$program
}

sub TVProgram::weekDaysTime{
	my$ctime=shift||time;
	my$oneday=24*60*60;
	my$time=$ctime-((6+(localtime)[6])%7+1)*$oneday;

	+sub{+$time+=$oneday}
}

sub TVProgram::weekDays{
	#      $weeks 
	#   $chan,    $time

	my$args={@_};
	my$chan=delete($$args{chan})||return;
	my($update, $town, $time)=delete@$args{'update', 'town', 'time'};
	my$weeks=delete$$args{weeks}||1;

	my$timer=&TVProgram::weekDaysTime($time);
	my($counter,$current)=$weeks*7;

	+sub{
		return undef unless $counter--;
		$current=&$timer;
		return +{
				time=>$current,
				program=>+&TVProgram::get(
								chan=>$chan,
								time=>$current,
								param=>{
									host=>$TVProgram::host
								},
								town=>$town,
								update=>$update
							)
		}
	}
}

sub TVProgram::getPrograms
{
	my$args={@_};
	my$sock=&TVProgram::getPage(%$args)||return +[];

	&TVProgram::skipHeaders($sock);
	&TVProgram::fgoto($sock,
		sub{+-1!=index$_,'<table border="0" cellspacing="1" cellpadding="0">'});
	my@page=&TVProgram::fstrgoto($sock, sub{+-1!=index$_,'</table>'});
	close $sock;

	my($program, $cur)=[];
	while(@page)
	{
		$cur=&TVProgram::getOneProg(\@page);
		$$cur{title}=~s/^<.*?>(.*?)<\/.+?>.*$/$1/ios;
		push@$program,$cur;
	}

	+$program
}

sub TVProgram::select_all{
	my(@cur,%res);

	open CHAN,'<'.$TVProgram::chan;
		#flock CHAN,4;
			scalar<CHAN>;
			while(<CHAN>)
			{
				chomp;
				@cur=split/\t/o,$_,3;
				@res{$cur[1]}={
					town=>$cur[0],
					title=>$cur[1]
				}
			}
		#flock CHAN,8;
	close CHAN;

	+\%res
}

sub TVProgram::getInfo{
	my$id=shift;
	my(@cur,%res,$fond);

	open CHAN,'<'.$TVProgram::chan;
		#flock CHAN,4;
			scalar<CHAN>;
			while(<CHAN>)
			{
				chomp;
				@cur=split/\t/o,$_,3;
				if($cur[1] eq $id)
				{
					@res{'town', 'id', 'title'}=@cur;
					$fond=1;
					last
				}
			}
		#flock CHAN,8;
	close CHAN;

	return undef unless $fond;
	+\%res
}

# .   .
#        .

sub TVProgram::get
{
	my$args={@_};
	my$update=delete$$args{update};
	my($chan, $time)=@$args{'chan', 'time'};
		#   UNIX_TIMESTAMP (10 )
		#   ,   
		#   http://tv.rambler.ru   channel_id
		#  ,         
		#   (  ,   : 94).
		#    "town".
	my$program;
	my$date=$$TVProgram::timer{"_$time:Y-m-d"};

	if($update||!($program=&TVProgram::load($chan, $date)))
	{
		$program=&TVProgram::getPrograms(%$args);
		&TVProgram::save($chan, $date, $program);
	}

	return $program;
}

1;